-- name: AgeVerificationMod v2.2
-- description: Implements an age-verification system in sm64ex-coop by testing the attention span of each player upon joining. Players must hold L, R, Z, A, and B continuously while also pressing and releasing D-Pad Left regularly. The duration is randomly set between 60 and 120 seconds. Moving, releasing any button, or failing to press/release D-Pad Left within 3 seconds will reset the timer and restart the test.

local maxTimeLeft = math.random(1800, 3600)
local maxInfoTime = 30
local verified = false
local timeLeft = nil
local infoTime = nil
local timer2 = 0
local isBtnZdown = false
local posX, posY, posZ = nil, nil, nil
local initialized = false
local timerAudioPlaying = false

local audioIntro = audio_stream_load("intro.mp3")
local audioTimer = audio_stream_load("timer.mp3")
local audioFail = audio_stream_load("fail.mp3")
local audioSuccess = audio_stream_load("success.mp3")

function play_custom_audio(customAudio, fileName)
    audio_stream_stop(audioTimer)
    customAudio = audio_stream_load(fileName)
    audio_stream_set_speed(customAudio, 44100, 1.0, true)
    audio_stream_play(customAudio, false, 1)
end

function on_mario_update(m)
    if m.playerIndex == 0 then
        if initialized == false then
            timeLeft = maxTimeLeft
            infoTime = maxInfoTime
            posX = m.pos.x
            posY = m.pos.y
            posZ = m.pos.z
            initialized = true
            audioIntro = audio_stream_load("intro.mp3")
            audio_stream_set_speed(audioIntro, 44100, 1.0, true)
            audio_stream_play(audioIntro, false, 0.5)
        end
        if not verified then
            infoTime = infoTime - 1
            
            if (m.controller.buttonDown & L_JPAD) ~= 0 then
                timer2 = isBtnZdown and (timer2 + 1) or 0
                isBtnZdown = true
            else
                timer2 = isBtnZdown and 0 or (timer2 + 1)
                isBtnZdown = false
            end
        
            if infoTime < 1 then
                infoTime = maxInfoTime
                if timeLeft == maxTimeLeft then
                    djui_chat_message_create("\\#ff8080\\You need to verify your age to play on this server!\nTo pass the attention span test, don't move and keep L, R, Z, A, and B pressed continuously. Also, keep pressing and releasing D-Pad Left. If D-Pad Left stays the same (pressed or unpressed) for 3 seconds, the timer will reset.\nRed text means you're doing it wrong; green means you're doing it right!")
                    timerAudioPlaying = false
                else
                    djui_chat_message_create("\\#80ff80\\Good! Keep holding down those buttons and make sure you press and release D-Pad Left within 3 seconds to avoid a reset!")
                    if timerAudioPlaying == false then
                        timerAudioPlaying = true
                        play_custom_audio(audioTimer, "timer.mp3")
                    end
                end
            end
            
            if (timer2 > 90) or (posX ~= m.pos.x or posY ~= m.pos.y or posZ ~= m.pos.z) then
                reset_timer(m)
            else
                if (m.controller.buttonDown & L_TRIG) ~= 0 and (m.controller.buttonDown & R_TRIG) ~= 0 and (m.controller.buttonDown & A_BUTTON) ~= 0 and (m.controller.buttonDown & B_BUTTON) ~= 0 and (m.controller.buttonDown & Z_TRIG) ~= 0 then
                    timeLeft = timeLeft - 1
                    posX = m.pos.x
                    posY = m.pos.y
                    posZ = m.pos.z
                else
                    reset_timer(m)
                end
                
                if timeLeft < 1 then
                    verified = true
                    djui_chat_message_create("\\#ff40ff\\AGE-VERIFICATION SUCCESSFUL!\n\\#ff40ff\\YOU CAN MOVE FREELY NOW!")
                    djui_popup_create("\\#ff40ff\\AGE-VERIFICATION SUCCESSFUL!\n\\#ff40ff\\YOU CAN MOVE FREELY NOW!", 1)
                    play_custom_audio(audioSuccess, "success.mp3")
                end
            end
        end
    end
end

function reset_timer(m)
    timerAudioPlaying = false
    if timeLeft < maxTimeLeft then
        djui_chat_message_create("\\#ff4040\\TOO BAD!\n\\#ff8040\\TIMER HAS BEEN RESET!")
        djui_popup_create("\\#ff4040\\TOO BAD!\n\\#ff8040\\TIMER HAS BEEN RESET!", 1)
        play_custom_audio(audioFail, "fail.mp3")
    end
    timeLeft = maxTimeLeft
    posX = m.pos.x
    posY = m.pos.y
    posZ = m.pos.z
end

function on_before_set_mario_action(m, a)
    if m.playerIndex == 0 and not verified then return 1 end
end

function on_allow_interact(m, o, i)
    if m.playerIndex == 0 and not verified then return false end
end

function on_chat_message(m, s)
    if m.playerIndex == 0 and not verified then return false end
end

function on_allow_hazard_surface(m, i)
    if m.playerIndex == 0 and not verified then return false end
end

function on_before_phys_step(m, i)
    if m.playerIndex == 0 and not verified then return 9 end
end

hook_event(HOOK_MARIO_UPDATE, on_mario_update)
hook_event(HOOK_BEFORE_SET_MARIO_ACTION, on_before_set_mario_action)
hook_event(HOOK_ALLOW_INTERACT, on_allow_interact)
hook_event(HOOK_ON_CHAT_MESSAGE, on_chat_message)
hook_event(HOOK_ALLOW_HAZARD_SURFACE, on_allow_hazard_surface)
hook_event(HOOK_BEFORE_PHYS_STEP, on_before_phys_step)
